#include <Ethernet.h>
#include <EthernetUdp.h>
#include <Wire.h>
#include <BME280I2C.h>
#include <PubSubClient.h>
#include <Dns.h>

BME280I2C bme;

uint16_t airQuality;
int val = 0;
float Temperatur;
float Humidity;
float Pressure;

// Netzwerk-Einstellungen
// MAC-Adresse darf nur einmal im Netz vohanden sein
// Fuer jedes Geraet aendern!!
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0x1A };

// Port fur Daten-Empfang
unsigned int localPort = 5010;

char msg[50];
// IP Adresse Loxone-MiniServer
IPAddress RecipientIP;

// Port Loxone-MiniServer... An diesen Port werden die Daten gesendet
unsigned int RecipientPort = 5010;

// Buffer fuer Daten-Empfang
char packetBuffer[UDP_TX_PACKET_MAX_SIZE];

// Start der UDP-Instanz
EthernetUDP Udp;


// DNS
DNSClient dnClient;

// Initialize the Ethernet server library
// with the IP address and port you want to use
// (port 80 is default for HTTP):
EthernetServer server(80);

void sendUDP(String);
void checkUDP();
void clearAll();
void MQTTreconnect();
void MQTTcallback(char* , byte* , unsigned int );


//MQTT
const char *MQTT_BROKER = "loxberry";
IPAddress mqttbroker;
EthernetClient ethClient;
PubSubClient MQTTclient(mqttbroker, 1883, MQTTcallback, ethClient);


void setup() {
	Wire.begin();
	Serial.begin(9600);
	//serset w5100
	Ethernet.begin(mac);
	if (!Ethernet.localIP()) {
		Serial.println("dhcp failed");
		Serial.println(Ethernet.linkStatus());
	} else {
		Serial.print("IP is ");
		Serial.println(Ethernet.localIP());
	}
	Udp.begin(localPort); // Start UDP
	sendUDP("Raumluft - aktiv"); // send UDP Ready

	// start the Ethernet connection and the server:
	server.begin();
	Serial.println("server is at ");
	Serial.println(Ethernet.localIP());

	if (!bme.begin()) {
		Serial.println("Could not find a valid BME280 sensor, check wiring!");
		while (1);
	}

	//DNS Client starten
	dnClient.begin(Ethernet.dnsServerIP());

	//MQTT Client
	MQTTclient.setServer(MQTT_BROKER, 1883);


	//Serial leeren
	clearAll();

	Serial.println("Warte auf UDP-Befehl");

}

void loop() {
	String aq = String(airQuality);
	String temp = String(Temperatur, 2);
	String hum = String(Humidity, 2);
	String press = String(Pressure / 100, 2);

	bme.read(Pressure, Temperatur, Humidity);

	// schaut on ein UDP Befehl empfangen wurde
	checkUDP();

	if (!strcmp(packetBuffer, "000")) {
		Serial.print("CO2 = ");
		Serial.print(airQuality);
		Serial.print(" ");
		Serial.println("[ppm]");

		// Wert wird auf 3000ppm begrnezt
		if (airQuality > 3000) {
			aq = "3000";
		} else {
			aq = airQuality;
		}
		sendUDP(aq);
	}

	if (!strcmp(packetBuffer, "001")) {
		sendUDP(temp);
	}

	if (!strcmp(packetBuffer, "002")) {
		sendUDP(hum);
	}
	if (!strcmp(packetBuffer, "003")) {
		sendUDP(press);
	}
	clearAll();

	// MQTT
	if (!MQTTclient.connected()) {
		MQTTreconnect();
	}
	MQTTclient.loop();

	snprintf(msg, 50, "Alive since %ld milliseconds", millis());
	Serial.print("Publish message: ");
	Serial.println(msg);
	MQTTclient.publish("/home/data", "Hello World");


	// listen for incoming clients
	EthernetClient client = server.available();
	if (client) {
		Serial.println("new client");
		// an http request ends with a blank line
		boolean currentLineIsBlank = true;
		while (client.connected()) {
			if (client.available()) {
				char c = client.read();
				//Serial.write(c);
				// if you've gotten to the end of the line (received a newline
				// character) and the line is blank, the http request has ended,
				// so you can send a reply
				if (c == '\n' && currentLineIsBlank) {
					// send a standard http response header
					client.println("HTTP/1.1 200 OK");
					client.println("Content-Type: text/html");
					client.println("Connection: close");
					client.println();
					client.println("<!DOCTYPE HTML>");
					client.println("<html>");
					// add a meta refresh tag, so the browser pulls again every 5 seconds:
					client.println(
							"<meta http-equiv=\"refresh\" content=\"10\">");
					// output the value of each analog input pin

					client.println("Current Humidity = "); //Prints information within qoutation
					client.println(hum); //Prints the Humidity read from the DHT11 on PIN 5
					client.println("% ");
					client.println("Temperature = ");
					client.println(temp); //Prints the temperature read from the DHT11 on PIN 5
					client.println("C ");
					client.println("Luftdruck = ");
					client.println(press);
					client.println(" ");
					client.println("[hPa]");

					client.println("</html>");
					break;
				}
				if (c == '\n') {
					// you're starting a new line
					currentLineIsBlank = true;
				} else if (c != '\r') {
					// you've gotten a character on the current line
					currentLineIsBlank = false;
				}
			}
		}
		// give the web browser time to receive the data
		delay(1);
		// close the connection:
		client.stop();
		Serial.println("client disonnected");

	}
}

//// Module ////
// Serial-Speicher loeschen
void clearAll() {
	// Paket-Buffer leeren
	for (int i = 0; i < UDP_TX_PACKET_MAX_SIZE; i++)
		packetBuffer[i] = (char) 0;
}

// empfangene UDP-Befehle auswerten
void checkUDP() {
	// pruefen ob Daten vorhanden sind
	int packetSize = Udp.parsePacket();
	if (packetSize) {
		Udp.read(packetBuffer, packetSize);
		Serial.print("Packet Content: ");
		Serial.println(packetBuffer);
		RecipientIP = Udp.remoteIP();
		Serial.print("Remote IP: ");
		Serial.println(RecipientIP);
	}
	delay(10);
}

// UDP-Befehl senden
void sendUDP(String text) {
	Udp.beginPacket(RecipientIP, RecipientPort);
	Udp.print(text);
	Udp.endPacket();
	delay(10);
}

void MQTTreconnect() {
	while (!MQTTclient.connected()) {
		Serial.print("Reconnecting...");
		if (!MQTTclient.connect("BME280")) {
			Serial.print("failed, rc=");
			Serial.print(MQTTclient.state());
			Serial.println(" retrying in 5 seconds");
			delay(5000);
		}
	}
}

void MQTTcallback(char* topic, byte* payload, unsigned int length) {
    Serial.print("Received message [");
    Serial.print(topic);
    Serial.print("] ");
    char msg[length+1];
    for (int i = 0; i < length; i++) {
        Serial.print((char)payload[i]);
        msg[i] = (char)payload[i];
    }
    Serial.println();

    msg[length] = '\0';
    Serial.println(msg);

    if(strcmp(msg,"on")==0){
        digitalWrite(13, HIGH);
    }
    else if(strcmp(msg,"off")==0){
        digitalWrite(13, LOW);
    }
}
